//  (C) Copyright John Maddock 2021

//  Use, modification and distribution are subject to the
//  Boost Software License, Version 1.0. (See accompanying file
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org/libs/config for more information.

//  MACRO:         BOOST_NO_CXX14
//  TITLE:         C++14
//  DESCRIPTION:   C++14 non-conformance

#include <boost/config/assert_cxx14.hpp>

namespace boost_no_cxx14 {

int test()
{
  return 0;
}

}
