//          Copyright Nikita Kniazev 2022.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#include <boost/config.hpp>

#ifndef BOOST_LIBSTDCXX_VERSION
# error "Not libstdc++"
#endif

int main() {}
